@@ -346,6 +346,18 @@ The lockspace handling code
 
 The initctl client code
 
+=item Sys::Virt::Error::FROM_CGROUP
+
+The cgroups code
+
+=item Sys::Virt::Error::FROM_IDENTITY
+
+The identity management code
+
+=item Sys::Virt::Error::FROM_ACCESS
+
+The access control code
+
 =back
 
 =head2 ERROR CODE CONSTANTS
@@ -708,6 +720,10 @@ The SSH operation failed
 
 The resource requested is already in use
 
+=item Sys::Virt::Error::ERR_ACCESS_DENIED
+
+Not authorized to perform the operation
+
 =back
 
 =head1 AUTHORS
